;--------------------------------------------------------------------
;
;
; BIOS CMOP-244-V1
; 25.04.2024	17:00
;
; entwickelt aus:
; MOPPEL-BIOS fr CP/M Rel. 2.2  V 12.5    30.10.85
; (C) Rheinhard Gssler 85
;
; EDITOR	Notepad++, TAB=6
;
;
;
; Stand:
;
;	13.06.2024	IO-Byte Bit 1,2 Console umgestellt	BIOS CMOP-244-V1-1 #
;	25.04.2024	Umstellung auf IDE-Interface		BIOS CMOP-244-V1
;--------------------------------------------------------------------
;	25.10.2024	IDE/CF-Init nach BIOS-Meldung		BIOS CF-V243-24a
;	21.04.2024	Baudrate Port f.87erV24 berichtigt
;			IO-Byte = bit2-7 DIP-Schalter
;			! FIFO fuer COM1 Fehlerhaft
;			KERMIT COM1 RDR/PUN bis 9600Bd OK
;	17.04.2024	IDE mit eigenem DPB			BIOS CF-V243-24
;	15.04.2024  Fehlerbereinigung				BIOS CF-V243-23
;			A=CF,B=SS,C=DS,D=IDE	sieht gut aus ! ;-)))
;
;			IDE aktiviert 				BIOS CF-V243-22
;			IDE integrieren als LW D		BIOS CF-V243-21
;	14.04.2024	ALV/CSV neu berechnet, 			BIOS CF-V243-20
;			Softsystem auf F800h angehoben	BIOS CF-V243-10c
;			Track auf Word umgestellt		BIOS CF-V243-10a
;			LW A mit 8MB !
;	07.04.2024  V24 auf19200Bd
;	17.03.2024	Stack zurueck auf 100h
;				/ Floppy-Parameter loggen
;				/ SoftSystem auf F500h
;				/ ITT3030 zurueck auf 70track wg. Prof
;	16.03.2024	STACK von 100h nach FA80 gelegt Fehler bleibt
;	14.03.2024	FIX # Dateien lesen > 1024 Records OK ! schreiben nur bis 1024 Records
;	---------------------------------------------------------------
;	24.02.2024	IO-Byte Bit 1-7 fuer BIOS
;			bit0 fuer Tastaturumschaltung SID/SOD
;			Einsatz NSC 800 fuer TurboPascal
;	19.02.2024	DS-Format DPB berichtigt
;			phy.LW D als 80Tr. SS Moppelformat hinzugefuegt
;	05.02.2024	Test mit grosser Datei erfolgreich
;	05.02.2024	Berichtigungen (Berechnung Seite) nur Track 0-70d OK
;	04.02.2024	Einbau ITT3030 DS Format
;			70Tr., DD, DS 16Sek.a256Byte Tr. 1-79 Side 0 Tr
;	---------------------------------------------------------------
;	04.02.2024	Grundlage BIOS 223-2 vom 13.01.2024
;	13.01.2024	Device Tabelle angepasst CON= COM1, COM2, COM3
;	07.01.2024	Bug in COM2 Input-Status berichtigt
;	21.11.2023	Bios bis f600h erweitert (Softsystem)
;			MV24 in Geraetetabell angepasst (1. 87er V24-Karte)
;
;	16.11.2023	IO-Byte FCh DFUe auf V24 CF-Karte 115kByte
;	15.11.2023	Pflege Gertetabell
;
;	18.02.2023	IO-Byte fest auf 0
;	19.04.2022  	Zurucke auf 8085 Simulator IDE
;			(io.mac integriert)
;	19.04.2022  	Workaround auf M80/L80 YAZE angepasst
;	13.05.2019	DIP-Schalter auf Multi-IO-Karte lesen und als IO-Byte speichern
;	08.05.2019  	Zuordnung CO-Routinen neu verteilt (tty, com1, com2, cv24)
;	11.11.2018	DPB 40Tr. DiskSize berichtigt				OK
;	28.10.2018	DPB fuer 40Track Laufwerk		#V18.13
;	20.02.2018	Multi-IO Karte eingebaut		#V18.12	OK
;			IO-Byte ueber DIP-Schalter einstellbar
;	17.02.2018	Quelltext auf TASM umgestellt		#V18.1	OK
;			notepad++ auf Tab=6
;
;
;*START IO.MAC*******************************************************
;
;include	IO231.MAC	; IO-Adressen
;
;--------------------------------------------------------------------
;
; Belegung 	IO-Adressen
; fuer Monitorversionen x.5
;
; Stand:	19.02.2022 - Workflow geaendert
;			 	 Assembler M80 unter Yaze
;
;		28.01.2018
;
;--------------------------------------------------------------------
;
; Multi-IO-Karte
;
;	1 x Timer
;		- Baudraten V24
; 	2 x V24
;		- COM1
;		- COM2
;	2 x PIO
;		- 8 DIP-Schalter fuer IO-Byte
;		- Systemuhr
;		- Centronis
;
; PIO1 Centronics und UNI-IO Kanal
;
pio1a		equ	03h		; Centronic's
pio1b		equ	13h		; frei
pio1c		equ	23h		; Steuerung Centronics
pio1s		equ	33h		; Steuerregister
bap1		equ	0abh		; 1010 1011
					; Kanal A Modus 1 Ausgabe, Kanal B Modus 0 Eingabe
					; Kanal C Eingabe
;
; PIO2 IO-Byte und Systemuhr
;
pio2a		equ	43h		;
pio2b		equ	53h		; IO-byte DIP-Schalter			<--
pio2c		equ	63h		; RTC 58321 Control
pio2s		equ	73h		; Steuerregister
bap2		equ	82h		; 1000 0010
bap2rdu	equ	92h		; 1001 0010
;
;
iobyteg	equ	pio2b		; IO-Adresse DIP-Schalter
;
;
;	Bit7 6 5 4 3 2 1 0
;	   : : : : : : : :
;        : : : : : : : + - 0= Moppel-Tastatur 	1= SID/SOD 1200Bd
;        : : : : : : + - - 0= CO Moppel Video	1= CP/M COM2
;        - - - - - - - - - CP/M IOByte
;
;
; Timer
;
tim0		equ	83h		; BD-Rate fuer com1
tim1		equ	93h		; BD-Rate fuer com2
tim2		equ	0a3h		; frei
tims		equ	0b3h		; Steuerregister
					; Timer0-2 als BCD-Zaehler, Rechteckgenerator,
					; erst Timewert-Low, dann Timewert-High laden
bati0		equ	37h		; 0011 0111
bati1		equ	77h		; 0111 0111
bati2		equ	0b7h		; 1011 0111
;
; V24
;
com1s		equ	0c3h		; Steuerregister
com1d		equ	0d3h		; Daten com1
bacom1	equ	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)		;
;
com2s		equ	0e3h		; Steuerregister
com2d		equ	0f3h		; Daten com2
bacom2	equ	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
;
;
;
;-------------------------------------------------------------------
;
; CF-Karte mit ATMega 644
;
cf_a		equ 	0cah		; Daten Ein-Ausgabe
cf_b		equ	0dah		; Daten Ausgabe B
cf_c		equ	0eah		; Daten Steuerung C
cf_ctl	equ	0fah		; Steuerwort
;
cf_ba		equ	0c1h		; Betriebsart
					; Kanal A bidirektional, Handshake Kanal C
;					; Kanal B als Ausgabe
;
;--------------------------------------------------------------------
;
; 1. 87er V24-Schnittstellenkarte
; als allgemeine DFUe- Schnittstelle
;
; ACIA 6850
;
com0s		equ	0e9h		; ACIA 6850 Control/Statusregister
com0d		equ	0f9h		;           Daten lesen/schreiben
;
; Mode-Funktionen
; Auswahl der Schnittstellen
;
;	Bit7 6 5 4 3 2 1 0
;	   : : : : : : : :
;        : : : : : : : + - /Relais 0 (0 = Relais aktiv)
;        : : : : : : + - - /Relais 1 (0 = Relais aktiv)
;        : : : : : + - - - Mod 2 = (20mA) (1= Ausgang aktiv)
;        : : : : + - - - - Mod 3 = (v24)
;	   : : : + - - - - - Mod 4 = (DTR)
;	   : : + - - - - - - Mod 5 = (PRT)
;	   0 0 - - - - - - - = 20mA
;	   0 1 - - - - - - - = V24
;	   1 0 - - - - - - - = PRT
;        1 1 - - - - - - - = Kassette
;
;
;	z.B. 	Mod 4Bh an cs_modex	= V24
;		Mod 43h an cs_modex	= V24 Printer an rckwertigen Buchse
;
; Modus Wahlschalter
;
cs_mode	equ	0a9h		; Modus (Eingangswahlschalter)
cs_bell	equ	0b9h		; Bell  (Schnarre)
;
; Timer 8253
;
timer_0s	equ	0d9h		; Timer 8253 Controlregister
timer_00	equ	89h		;        Zaehler 0
timer_01	equ	99h		;	   Zaehler 1 * nicht benutzt
timer_02	equ	0c9h		;	   Zaehler 2 * nicht benutzt
;
;
;--------------------------------------------------------------------
;
; 2. 87er V24-Schnittstellenkarte
; als Console anstelle der Moppeltastatur und Bildschirm
;
; ! Adressbereich identisch mit Paralellschnittstelle !
;
cons		equ	0e8h		; ACIA 6850 Control/Statusregister
cond		equ	0f8h		;           Daten lesen/schreiben
;
cs_mode1	equ	0a8h		; Modus (Eingangswahlschalter)
cs_bell1	equ	0b8h		; Bell  (Schnarre)
;
timer_1s	equ	0d9h		; Timer 8253 Controlregister
timer_10	equ	88h		;        Zaehler 0
timer_11	equ	98h		;	   Zaehler 1 * nicht benutzt
timer_12	equ	0c8h		;	   Zaehler 2 * nicht benutzt
;
startbd	equ	0020		; Startbaudrate 9600Bd
;
;--------------------------------------------------------------------
;
; RAM-Bank Umschaltung fuer CP/M Betrieb
;
bank		equ	24h		; Port fuer Bankumschaltung
;
;					; 81h an 24h 0000-3fffh EPROM/VID
;					;            4000-ffffh RAM
;					; 01h an 24h 0000-ffffh RAM
;
;--------------------------------------------------------------------
;
; FDC-Karte
;
fdc_sel	equ	40h		; Floppy Selekt-Byte
fdc_ctl	equ	48h		; WD1720 Controlregister
fdc_sp	equ	49h		; Spurregister
fdc_sek	equ	4ah		; Sektorregister
fdc_dat	equ	4bh		; Datenregister
;
;
;--------------------------------------------------------------------
;
; Parallelschnittstelle (keine ECB-Karte)
;
par_ctl	equ	0d8h		; 8255 Control/Statusregister
par_pa	equ	0a8h		;      Port A
par_pb	equ	0b8h		;      Port B
par_pc	equ	0c8h		;      Port C
;
;
;--------------------------------------------------------------------
;
;
; Echtzeituhr
;
uhr_ctl	equ	19h		; 5832 Controlregister
uhr_dat	equ	18h		;      Datenregister
;
;
;--------------------------------------------------------------------
;
; Thermodrucker
;
therm_d	equ	38h		; Daten
therm_ctl	equ	39h		; Controlregister
;
;
;--------------------------------------------------------------------
;
; Eprom brenner
;
prom_ctl	equ	0bh		; Controlregister
prom_adrl	equ	09h		; Adressen untere Haelfte
prom_adrh	equ	0ah		;          obere Haelfte
prom_dat	equ	08h		; Daten
;
;
;--------------------------------------------------------------------
;
; Einzelschrittmodul
;
ssm_ff	equ	1ah		; Singlestep-Modul FlipFlop
;
;
;--------------------------------------------------------------------
;
; Videokarte
;
vid_dspen	equ 	2ah		; Format/Blanking
					; (Umschaltung Zeichensatz/
					; Abfrage DiSPlayENable
vidctl	equ	2ah		;
vidreg	equ	28h
vidat		equ	29h
					;
lin		equ	50h		; 80d Zeichen pro zeile
lininv	equ	0ffb0h	; - 80d + 1
;
;
;--------------------------------------------------------------------
;
; Hex-display
;
led_zeil	equ	00h		; LED-Zeile
hex_d1	equ	10h		; DIGIT 1
hex_d2	equ	20h		; DIGIT-2
hex_d3	equ	30h		; DIGIT-3
hex_d4	equ	40h		; DIGIT-4
hex_d5	equ	50h		; DIGIT-5
hex_d6	equ	60h		; DIGIT-6
hex_d7	equ	70h		; DIGIT-7
outpor	equ	80h		; DIGIT-7 (+10h)
;
;
;--------------------------------------------------------------------
;
; Hex-Tastatur
;
comand	equ	0bch		; CMD-Keys
hexhi		equ	0dch		; Hex 8..F
hexlo		equ	0ech		; Hex 0..7
;
nxtnr		equ	0h		; NXT-Spalte
fctnr		equ	2h		; FCT-Spalte
adrnr		equ	4h		; ADR-Spalte
runr		equ	6h		; RUN-Spalte
diodnr	equ	7h		; Ge-Diode (zur Unterscheidung
;
;
;--------------------------------------------------------------------
;
; HEX-/ASCII-Tastatur)
;
; ASCII-Tastatur			Adressbereich 0ch bis fch
;
ascii		equ	0fch		; oberste Adresse
;					;
;
;--------------------------------------------------------------------
;
;*Ende IO.MAC*******************************************************
;
;
;--------------------------------------------------------------------
;
; Moppel Monitor Routinen
;
ci_m		equ	43h		; Console Input
co_m    	equ	49h		; Console Output
csts_m  	equ	52h		; Console Status
lomo    	equ	4fh		; Lister Output
curson	equ	1036h		; Curson
time		equ	61h		; Uhrzeit lesen und im ASII-Buffer bereitstellen
settim	equ	79h		; Uhrzeit stellen (HL)Std,min, (BC)Wochentag, (DE)Monat,Jahr
;
timbuf	equ	2fe9h		; ASCII-Buffer fuer Uhrzeit
;
ioflag	equ	2fc8h		; LO Select
;
stack0	equ	2950h		; Bank 0 Stack
;
;------------------------------------------------------------------------------
;
;      60-K-Version
;
dip_io	equ	pio2b		; DIP-Schalter auf Multi-IO
baud		equ	0020h		; 9600 Bd fuer Muti-IO-Karte
;					; fuer KERMIT
;
int55		equ	002ch		; IRQ Sprungleiste
int65		equ	0034h		;
int75		equ	003ch		;
;
bios  	equ	0ea00h	; Start BIOS
ccp   	equ	0d400h	; Start CCP
bdos  	equ	0dc06h	; Start BDOS
;
stack2	equ	0faffh	;
;
verbuf	equ	0fb00h	;
;
buffer	equ	0fc00h	; 1kByte fuer Disk Rd/Wr
nsects	equ	2ch		; Anzahl Sector fr CPML
offset	equ	04h		; Start bei Track 4
vers		equ	22h		; Versions-Nr.
;
buff  	equ	0080h		; default Buffer Adress
stack 	equ	0100h		; default STACK
ndisks	equ	04h		;
iobyte	equ	0003h		;
cdisk 	equ	0004h		; Adr 4 fr last Drive
;
;====================================================================
;
;		aseg

		org	bios
;
       	jmp	boot		; Einsprungleiste
wboote:	jmp	wboot		;
;
; IO-Geraete
;
       	jmp	csts		; Status Console
       	jmp	ci		; Input  Console
       	jmp	co		; Output Console
       	jmp	list		; Output Lineprinter
       	jmp	punch		; Output Puncher
       	jmp	reader	; Input  Reader
;
; Floppy
;
       	jmp	home		;
       	jmp	seldsk	; Select Disk
       	jmp	settrk	; Set    Track
       	jmp	setsec	; Set    Sector
       	jmp	setdma	; Set    Disk-Buffer
       	jmp	read		; Read   Disk *
       	jmp	write		; Write  Disk *
       	jmp	lsts		; Status Lineprinter
       	jmp	sectra	; nicht benutzt
;
; BIOS Erweiterung V24
;
; V24 Moppel 87er Hardware	WBOOT + 30h
;
		jmp	mv24bd	; (DE) Baudrate einstellen
		jmp	mv24ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	mv24rx	; (A) Byte empfangen, wartet auf Zeichen
		jmp	mv24rxz	; (A) Byte empfangen, abbruch nach 10s mit Control-Z (1ah)
		jmp	mv24tx	; (C) Byte senden
		jmp	mv24txz	; (C) Byte senden, abbruch wenn Geraet nicht bereit
;
; V24 auf CF-Karte
; * mit Erweiterungen fuer CF-Interface
;
		jmp	cv24ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	cv24rxb	; (A) Byte empfangen
		jmp	cv24txb	; (C) Byte senden
		jmp	cv24rxbz	; (C) Byte empfangen mit Timeout
		jmp	cv24txbz	; (C) Byte senden mit Timeout
;
; BIOS Erweiterung Echtzeituhr MSM 5832
;
		jmp	rdtime	; Uhrzeit auslesen (HL)= zeiger auf ASCII-Buffer
		jmp	wrtime	; Uhrzeit stellen (HL)Std,min, (BC)Wochentag,Kalendertag (DE)Monat,Jahr
;
; BIOS Erweiterung V24 auf Multi-IO-Karte
;
; 1.Schnittstelle COM1
;
		jmp	com1bd	; (DE) Baudrate einstellen
		jmp	com1ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	com1r		; (C) Byte senden
		jmp	com1t		; (A) Byte empfangen
		jmp	com1a		; (A) Byte empfangen mit Timeout
		jmp	com1rx	; wie com1a + FIFO
;
; 2.Schnittstelle COM2
;
		jmp	com2bd	; (DE) Baudrate einstellen
		jmp	com2ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	com2r		; (A) Byte empfangen
		jmp	com2t		; (C) Byte senden
		jmp	com2a		; (C) Byte empfangen mit Timeout
;
;
;====================================================================
;
;      DPBase
;
;--------------------------------------------------------------------
;
dpbase:	ds	0h		;
;
; LW A:	CF-Karte
dpe0:		dw	xlt0		; Translate table
		dw	0000h
		dw	0000h		; Sratch area
		dw	0000h
		dw	dirbuf	; Dir. Buffer
		dw	dpb3		; Parameter Block
		dw	csv3		; Check
		dw	alv3		; Allocation Vectors
;
; LW B:	80Tr-Laufwek
dpe1:		dw	xlt1
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb0
		dw	csv0
		dw	alv0
;
; LW C:	80Tr-Laufwerk
dpe2:		dw	xlt2
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb1
		dw	csv1
		dw	alv1
;
; LW D:	80Tr-Laufwerk
dpe3:		dw	xlt3
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb0
		dw	csv0
		dw	alv0
;
;
;====================================================================
;
;      Disk Parameter Block
;
;--------------------------------------------------------------------
;
; dpb0 fr 80 Track-Laufwerke
;
dpb0:		dw	20h 		; Sectors per Track
		db	04h 		; Block shift
		db	0Fh 		; Block Mask	BLS = 2048
		db	01h  		; Extnt Mask
		dw	97h 		; Disk Size-1
		dw	7fh 		; Directory max.
		db	0c0h		; Alloc0
		db	00h 		; Alloc1
		dw	10h 		; Check Size
		dw	4h  		; Offset
;
xlt0		equ	0h		;Sec.Translate Vec.;no Transl.Table
;
; dpb1 fr 80 Track-Laufwerke	; DS 256Byte/Sek. ITT3030
; Seite 2 auf Track 0-69 Side 1
;
dpb1:		dw   20h 		; log.Sectors per Track
		db   04h 		; Block shift
		db   0Fh 		; Block Mask	BLS = 2048
		db   0h  		; Extnt Mask
		dw   117h 		; Disk Size-1
		dw   7fh 		; Directory max.
		db   0c0h		; Alloc0
		db   00h 		; Alloc1
		dw   20h 		; Check Size
		dw   4h  		; Offset
;
xlt1   	equ	xlt0
;
;
; dpb2 fr 80 Track-Laufwerke	; DS 512Bte/Sek Prof#4
; BSH 4 BLM 15 EXM 0 DSM 384 DRM 127 AL0 0C0H AL1 0 OFS 3
;
dpb2:		dw	50h		; Sectors per Track
		db	04h		; Block shift
		db	0fh		; Block Mask	BLS = 1024
		db	0h 		; Extnt Mask
		dw	384		; Disk Size-1
		dw	127		; Directory max.
		db	0c0h		; Alloc0
		db	00h		; Alloc1
		dw	32h		; Check Size
		dw	3h 		; Offset
;
xlt2		equ	xlt0
;
;
; CF-Karte
;
; 8MB File auf CF-Karte
; Offset fr Sektoren sind im AVR-Teil einzurichten
;
; 32 Records a 128Byte
; 4kBlcke
; 512 Dir Eintrge
;     ohne Check (CF-Karte als "Festplatte")
; 4 Systemspuren
;
; Wichtig alv3 anpassen
; Parameter stehen im hinteren Bereich
; ab Soft-System
; alv3 = DSM+1 / 8
;
dpb3:		dw	32  		; Sectors per Track
		db	5  		; Block shift
		db	31  		; Block Mask	BLS = 4096
		db	1   		; Extnt Mask
		dw	2047  	; DSM Disk Size-1 = 8MB
		dw	511  		; Directory max.
		db	11110000b 	; Alloc0
		db	0	  	; Alloc1
		dw	0	  	; Check Size
		dw	1	   	; Offset
;
xlt3		equ	xlt0		; keine Sektorbersetzung
;
;
;====================================================================
;
;      Kalt- und Warmstart
;
;--------------------------------------------------------------------
;
boot:		call	curson	; Cursor einschalten
		mvi	a,01h		;
		out	24h		; Only Bank 1 select
		lxi	sp,stack	; Initialisierung STACK

		call	mioini	; Multi-IO-Karte
		call	mv24ini	; 1. 87er V24 Port e9h
;
;
set_iob:	in	iobyteg	; IO-Byte Schalter lesen
		ani	11111100b	; DFue auf V24 CF-Karte
;		xra	a		; * (auf Standart)
		sta	iobyte	;
;
;
; Systemmeldung ausgeben
;
		lxi	h,msg1	;
		call	msgout	;
;
		call	ideinit	; IDE Interface
;
clear:	xra	a		;
		sta	wrtflg	; Buffer ungueltig
		sta	cdisk		; Select Drive 0
		cma			;
		sta	aktdrv	; letztes Drive undefiniert
		lxi	h,0000h	;
		shld	track0	; trk-reg 0+1  undefiniert
		shld	track2	; trk-reg 2+3  undefiniert

gocpm:	mvi	a,0c3h	; Initialisierung Page 0
		sta	0000h		;
		sta	0005h		;
		lxi	h,wboote	;
		shld	1h		;
		lxi	h,bdos	;
		shld	6h		;
		lxi	b,buff	;
		call	setdma	;
		lda	cdisk		;
		mov	c,a		;
		jmp	ccp		;
;
;
;
;
wboot: 	lxi  	sp,buff
;
		lda  	wrtflg    	; wurde alter Buffer auf Disk
		ora	a         	; geschrieben ?
		jz   	load		; write alten Buffer auf Disk
		call 	puttrk    	; weiter mit Floppy
;
load:		xra  a
		cma
		sta wrtflg     	; Buffer ungueltig
; CCP und BDOS neu laden
; von IDE
;
		lxi	h,ccp		; Startadresse 60k CP/M
		shld	datadr	; - sichern
		mvi	a,0h		; Startspur = 0
		sta	trknr		; - sichern
		sta	lba+1		; als LBA + 1
		mvi	a,2h		; Startsektor = 2
		sta	secnr		; - sichern
		sta	lba		; als LBA
;
bootl:	call	read_sector	; Sektor schreiben
		lda	secnr		;
		inr	a		; Sektornummer hochzaehlen
		sta	secnr		;
		sta	lba		; als LBA
		cpi	11		; 11 Sektoren laden
		jc	bootl		; nein - weiter
		;
		jmp	gocpm		; zum ccp
;
;
msgout:	mov	a,m		;
		ora	a		;
		rz			; => Ende
		mov	c,a		;
		push	h		;
		call	co		;
		pop	h		;
		inx	h		;
		jmp	msgout	;
;
;====================================================================
;
;      Peripherie-Verwaltung
;
;	mv24..	= V24 auf Moppel 87er Hardware	9600Bd
;	com1/2	= Multi-IO-Karte
;     cv24..	= schnelle V24 auf CF-Karte		115000Bd
;
;--------------------------------------------------------------------
;
co:    	lda	iobyte	; CP/M : Console Output
       	ani	03h
       	lxi	h,cotab	; Startadr CO-Tab
       	jmp	decode

;
ci:    	lda	iobyte	; CP/M : Console Input
       	ani	03h
       	lxi	h,citab	; Startadr CI-Tab
       	jmp	decode
;
csts:  	lda	iobyte	; CP/M : Console Status
       	ani	03h
       	lxi	h,csttab
       	jmp	decode
;
;---
;
reader:	lda	iobyte	; CP/M : Reader Input
		rrc
		rrc
		ani	03h
		lxi	h,ritab	; Startadr Reader-Tab
		jmp	decode
;
;---
;
punch:	lda	iobyte	; CP/M : Puncher Output
		rrc
		rrc
		rrc
		rrc
		ani	03h
		lxi	h,potab	; Startadr Puncher-Tab
		jmp	decode
;
;---
;
list:		lda	iobyte	; CP/M : List Output
		rlc
		rlc
		ani	03h
		lxi	h,lotab	; Startadr List-Tab
		jmp	decode
;
;---
;
lsts:  	lda	iobyte	; CP/M : Lister Status
       	rlc
       	rlc
       	ani	03h
       	lxi	h,lsttab
       	jmp	decode
;
;
; IO-Tabelle auswerten
;
decode:	jz	found		; Setzt PC entsprechend IOBYTE
       	inx	h
       	inx	h
       	dcr	a
       	jmp	decode
found: 	mov	e,m		;Zieladresse auslesen
       	inx	h
       	mov	d,m
       	xchg
       	pchl			;Verzweigung
;
; Tabellen fuer IO-Geraete
;
;	COM1 fuer DFUe				9600Bd
;	COM2 fuer List				9600Bd
;	mv24 fuer Console/Drucker/DFUe	9600Bd
;	CV24 fuer DFUe				115kBd
;
;
; Consolen Output
;
cotab: 	dw	co_mon	; TTY	; Moppel Console
       	dw	co_mon	; TTY	; Moppel Console #com2t		; CRT	; Com2
       	dw	com1t		; BAT	; Com1
       	dw	cv24txb	; UC1	; CF-Karte
;
; Consolen Input
;
citab: 	dw	ci_mon	; TTY ; Moppel Console
       	dw	ci_mon	; TTY ; Moppel Console #com2r		; CRT	; Com2
       	dw	com1r		; BAT	; Com1
       	dw	cv24rxb	; UC1	; CF-Karte
;
; Consolen Status
;
csttab:	dw	cs_mon	; TTY	; Moppel Console
       	dw	cs_mon	; TTY	; Moppel Console #com2ist	; CRT	; Com2
       	dw	com1ist	; BAT ; Com1
       	dw	cv24ist	; UC1	; CF-Karte
;
; Reader
;
ritab:	dw	mv24rxz	; TTY	; 1.87erV24
		dw	com1a		; PTR	; Com1 mit Zeitabbruch
		dw	com1r		; UR1	; Com1
		dw	cv24rxb	; UR2	; CF-Karte
;
; Puncher
;
potab:	dw	mv24tx	; TTY	; 1.87erV24
		dw	com1t		; PTP	; Com1
		dw	com1t		; UP1	; Com1
		dw	cv24txb	; UP2	; CF-Karte
;
;
; List
;
lotab:	dw	co_mon	; TTY	; Moppel Console
		dw	com2t		; CRT	; com2
		dw	com1t		; LPT	; com1
		dw	cv24txb	; UL1 ; CF-Karte
;
; List Status
;
lsttab:	dw	mcosta	; TTY	; Moppel Console
       	dw	rest		; CRT ; Res.
       	dw	rest		; LPT ; Res.
       	dw	cv24ost	; UC1	; CF-Karte
;
;
;====================================================
;
;      Ausgabe ueber die serielle Schnittstelle
;	! keine Hardware dazu
;====================================================
;
prt:		push	h
		lxi 	h,0h
		dad 	sp		;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl			;Bank 0 Stack
		mvi 	a,81h
		out 	24h   	;Umschalten auf Bank 0

		push	d     	;Bank 1 Stack retten
		mvi 	a,80h 	;Printer waehlen
		sta 	ioflag
		call	lomo  	;Monitor-UP
		mvi 	a,0h  	;Printer inaktiv
		sta 	ioflag	;(wg.CO; sonst doppeltes LO)
		pop 	h     	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h   	;Umschalten auf Bank 1

		sphl	      	;alter Bank 1 Stack
		pop 	h
		ret
;
;
;====================================================
;
;      Ausgabe ueber die parallele Schnittstelle
;	! keine Hardware dazu
;====================================================
;
centr:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		mvi 	a,40h   	;Centronics waehlen
		sta 	ioflag
		call	lomo    	;Monitor-UP
		mvi 	a,0h    	;Centronics inaktiv
		sta 	ioflag  	;(wg.CO; sonst doppeltes LO)
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl			;alter Bank 1 Stack
		pop  	h
		ret
;
;
;
;====================================================
;
;      Monitor-Routinen CO, CI und CSTS
;
;====================================================
;
;
;      Ausgabe auf Bildschirm
;
co_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	co_m    	;Monitor-UP
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,01h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		ret

mcosta:	mvi  a,0ffh		; Status CO-MON
		ret
;
;
;      Eingabe von der ASCII-Tastatur
;
ci_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	ci	   	;Monitor-UP
		push	psw     	;wg. MOVID-5.3-CSTS
cilop:	call	csts  	;Monitor-UP
		jnz 	cilop   	;auf Loslassen warten
		pop 	psw
		mov 	c,a     	;temp. ablegen
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		mov 	a,c     	;Input zurueckholen
		rlc 	        	;CTL-Codes MOVID 5.3?
		jc  	ctlkey  	;ja: JMP
		mov 	a,c
		ret
;
ctlkey:	mov 	a,c     	;fehlende CTL-Codes erzeugen
		cpi 	8fh     	;HOME?
		jz  	mark    	;ja: JMP
		ani 	7fh
		sui 	20h
		ret
;
mark:		mvi 	a,27h   	;Hochkomma
		ret
;
;
;      Status der ASCII-Tastatur
;
cs_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	csts  	;Monitor-UP
		pop 	h       	;Bank 1 Stack zurueck
		mov 	c,a     	;Status retten
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		mov 	a,c     	;Status zurueckholen
		ret
;
; DIP-Schalter lesen und als IO-Byte setzen
;
;rdiobyte:	in	2bh		; DIP-Schalter einlesen
;		sta	iobyte	;
;		ret			;
;
;
;====================================================
;
;      Status Dummy
;
;====================================================
;
prtst: 	ds 0h
censta:	ds 0h
v24st: 	ds 0h
crtst: 	ds 0h
rest: 	ds 0h
uscn3s:	ds 0h
usri2s:	ds 0h
usri3s:	ds 0h
uspo2s:	ds 0h
uspo3s:	ds 0h
;
rdysta:	xra  a         ;Dummy Status: Device Ready
		dcr  a         ;A: FFh
		ret            ;ZERO off
;
;
;====================================================
;
;      Disketten-Management
;
;====================================================
;
; BIOS-Funktion Home
;		Set Track (C)=Track
;
; I:	(BC)	=	Track
; O:
;
home:  	lxi	b,0000h	; Track 0
settrk:	lxi	h,iotrk	; Trackbuffer laden
       	mov	m,c
		inx	h
		mov	m,b
       	ret
;
; BIOS-Funktion Set Sektor (C)=Sektor
;
setsec:	lxi	h,iosec	; Sectorbuffer laden
       	mov	m,c
       	ret
;
; BIOS-Funktion Set DMA	(BC)=DMA Adresse
;
setdma:	mov	l,c		; DMA - Adresse laden
       	mov	h,b
       	shld	iodma
       	ret
;
; BIOS-Funktion Set Drive (C)=Laufwerk
;			  (HL)= DPH
;
seldsk:	lxi	h,0000h	; Bei Fehler notwendig
       	mov	a,c
       	cpi	ndisks	; LW im Bereich ?
       	rnc
;
       	sta	iodrv		; LW in Buffer
       	add	a		; ;2
       	add	a		; ;4
       	add	a		; ;8
       	add	a		; ;16 (Laenge eines DPBs)
       	mov	l,a
       	lxi	d,dpbase
       	dad	d
       	ret
;
; BIOS-Funktion Sec Translate
; !im Moppel nicht bentigt
;
sectra:	mov	a,d
       	ora	e
       	jz	se1		; Test ob Xlate-Tab vorhanden
;
       	xchg
       	dad	b
       	mov	a,m
       	sta	iosec
       	mov	l,a
       	mvi	h,00h
       	ret
;
se1:   	mov	l,c
       	mov	a,c
       	sta	iosec
       	mvi	h,0h
       	ret
;
;------------------------------------------------------------
;
; calc - CP/M Sektorgroee auf Moppel Sektorgroee umrechnen
;
;	CP/M-Format 	32 Sektoren/Track a128Byte [Record]
;	Moppel-Format 	16 Sektoren/Track a256Byte
; I:	iosek		vom BDOS
; O:	(B) 		Laufwerk
;	(C)		Sektor
;	(DE)		Track
;
;	 (E)= Startsektor fr R/W Disk
;
calc:		lda	iosec
		rrc
		ani	0ch
		inr	a
		mov	c,a
		lda	iodrv
		mov	b,a
		lhld	iotrk
		xchg
		ret
;
; comp - Test ob Drive/Track/Sektor noch gleich ist
;
comp:		lda	aktdrv
		cmp	b
		rnz
		lhld	akttrk
		mov	a,l
		cmp	e
		rnz
		mov	a,h
		cmp	d
		rnz
		lda	aktsec
		cmp	c
		ret
;
; store - sicher Drive/Track/Sektor
;	  fr Disk-Routinen
;
store:	mov	a,b		;
		sta	aktdrv	;
		mov	a,c		;
		sta	aktsec
		xchg
		shld	akttrk
		ret
;
;


;
; BIOS-Funktion Sektor lesen
;
read:  	di			;
		call	calc		; <== Einsprung von BDOS
		call	comp
		jnz 	rload		;
;
r1rd:		call	bufcal	; Daten im Buffer, nur kopieren
		call	copyx
		xra 	a
		ret
;
; bufcal - Record auf 1kByte-Buffer abbilden
;
bufcal:	lxi	h,buffer
       	lda	iosec
       	ani	07h
       	rrc
       	mov	d,a
       	mvi	a,00h
       	rar
       	mov	e,a
       	mov	a,d
       	ani	7fh
       	mov	d,a
       	dad	d
       	xchg
       	lhld	iodma
       	xchg
       	mvi	b,80h		; Recordlaenge = 128 Bytes
       	ret
;
;
; rload - Test ob Buffer gltig ist (wrtflg)=1
;         wenn nein, alten Buffer zurckschreiben
;
rload: 	lda	wrtflg
       	ora	a
       	jz	r1load
       	call	puttrk	; alten Buffer sichern
		jc   	errio
;
; Buffer von Disk holen
;
r1load:	call	calc
       	call	store
       	call	gettrk	;
		jc   	errio
       	jmp	r1rd
;
;
; Copyx kopiert die Daten vom Buffer (FC00h) in DMA-Buffer
;			  und umgekehrt (DE-HL) getauscht
;
; (B)	Recordlnge
; (DE)	= Zieladress
; (HL)	= Quelladresse
;
copyx: 	mov 	a,m		;
		stax	d
		inx 	h
		inx 	d
		dcr 	b
		jnz 	copyx
		ret
;
; BIOS-Funktion Sektor schreiben
;
; I:	vorher alle SET-Befehle Drive, Track, Sektor, DMA
; O:	log.Sektor von BDOS DMA nach Buffer
;	erst wenn Buffer ungueltig ist erfolgt der
;	physikalische Laufwerkszugriff
;
write: 	di			;
		call	calc		; <== Einsprung von BDOS
		call	comp
		jnz 	wload
;
w1wr:		call	bufcal	; Daten im Buffer, nur kopieren
		xchg
		call	copyx		; Record in DMA Buffer
		mvi 	a,01h
		sta 	wrtflg	; Buffer wieder gltig
		xra 	a
		ret
;
;
wload: 	lda	wrtflg	;
		ora 	a
		jz  	w1load
		call	puttrk	; alten Buffer Sichern
		ana	a
		jnz 	errio
;
w1load:	call	calc
		call	store
		call	gettrk	; neuen Buffer holen
		ana 	a
		jnz 	errio
		jmp 	w1wr
;
; Fehlermeldung an BDOS (A)=01h
;
errio:	mvi	a,1h
		ora	a
		ret
;
;
;====================================================
;
;     GETTRK/PUTTRK (Bufferverwaltung)
;	Block lesen/schreiben 1KB-Buffer
;	mit CF-Interface
;
; Datenbergabe:
;	aktdrv <== iodrv BDOS SET-DISK
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec 1.Sektor in Moppelformat
;			(Umrechnung aus CP/M) <== BDOS SET-Sektor
;
;	wrtflg wird auf 0 gesetzt Buffer ungltig)
;
; interne Speicherzellen:
;	datadr	= aktuelle Bufferadresse
;	trknr	= laufende Spurnummer
;	secnr	= laufende Sektornummer
;
; Rckgabe:
;	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)	= 0 fr OK
;	(CY) 	= 1 Fehler lesen/schreiben
;
;====================================================
;
; Test ob Laufwerk A ausgewhlt wurde
;			dann zu den CF-Routinen
;
gettrk:	push	psw		;
		lda	aktdrv	;
		cpi	00h		; Laufwerk = A?
		jz	get_ide	; ==> zu CF-Routinen
		jmp	get_fl	; ==> nein weiter mit Floppy
;
;
puttrk:	push	psw		;
		lda	aktdrv	;
		cpi	00h		; Laufwerk = A ?
		jz	put_ide	; ==> zu CF-Routinen
		jmp	put_fl	; ==> nein weiter mit Floppy
;
;
; Floppy-Routinen vorbereiten
;	Buffer
;	Sektor
;	Laufwerksnummer fr FDC aufbereiten
;	Anzahl der zu kopierenden Sektoren
;
codbuf:	xra	a		; clear wrtflg -"Buffer leer"
		sta	wrtflg	;
		lxi	h,buffer   	;
		lda	aktdrv	;
		rlc			;
		ani	06h		;
		ori	18h       	; DD / 30ms / Side 0
		mov	b,a		;
		lda	aktsec   	;
		mov	e,a
		mvi	d,4h      	; 4 Sectoren
		ret
;
; Geaendert auf 70Tr.ITT3030
;
ds:		lda 	aktdrv	;
		cpi	02		; Test auf LW C mit DS ITT3030
		rnz			; nein
		mov	a,d		;
		sui	46h		; weiter mit Track 1-80 2.seite
		mov	d,a		;
		mov	a,b		;
		ori	20h		; Seite 1 einschalten
		mov	b,a		;
		ret			;
;
; Buffer mit 4 Sektoren laden
;
get_fl:	pop	psw		;
		call	codbuf    	; Reg vorbereiten
;
getlp: 	push	h
		push	d
		push	b
		lda 	akttrk	;
		mov 	d,a
		cpi	46h		; ab Track 80 auf Seite 1
		cnc	ds		; wenn LW C
		call	secrd
		pop 	b
		pop 	d
		pop 	h
		ana 	a
		jnz 	errx
		call	nxtsec
		jnz 	getlp
		xra 	a
		ret			; zurck zum Disk-Manager
;
;
; 4 Sektoren vom Buffer auf Floppy schreiben
;
put_fl:	pop	psw		;
		call	codbuf	;
putlp:	push	h
		push	d
		push	b
		lda 	akttrk
		mov 	d,a
		cpi	46h		; ab Track 80 auf Seite 1
		cnc	ds		;
		call	secwr
		pop 	b
		pop 	d
		pop 	h
		ana 	a
		jnz 	errx
		call	nxtsec
		jnz 	putlp
		xra 	a
		ret			; zurck zum Disk-Manager

nxtsec:	push	d         	; Reg fuer naechsten Sec setzten
		lxi	d,0100h
		dad	d         	; Bufferadr + 256 Bytes
		pop	d
		inr	e
		dcr	d         	; Sec-Zaehler - 1
nxtend:	ret
;
;
;
; Fehler an Diskmanager bergeben (CY)=1
;
errx:  	stc
		ret
;
;
;====================================================
;
;      Floppy-Routinen
;
;====================================================
;
; WD1770 Befehle
;
;	restor	equ	 00h  ;Pos.Track 00
;	seek  	equ	 10h  ;Pos.Track tt
;	stepin	equ	 50h  ;Pos.Track tt+1
;	rdsc  	equ	 80h  ;Read Sector ss
;	wrsc  	equ	0a2h  ;Write Sector ss (no Precomp.)
;	forst 	equ	0d0h  ;Force reset
;	force 	equ	0d8h  ;Force Immediate Interrupt
;	wrtrk 	equ	0f2h  ;Write Track
;
; FDC Port-Adressen
;
comreg      equ	48h		; FDC
trkreg      equ	49h		; FDC
secreg      equ	4ah		; FDC
datreg      equ	4bh		; FDC
status      equ	comreg
ctrl        equ	40h		; (FDC)-Drive-Select
;
;
		nop
;
intrd:	in	datreg
		stax	d
		inx 	d
		ret
;
;
intwr:	ldax	d
		out	datreg	;
		inx	d		;
		ret
;
;
inton:	push	b
		mvi 	a,1eh     	; 8085-RST5.5 enable
		mvi 	b,02h     	; NSC800-RSTC enable
		jmp 	intcte	;
intof:	push	b
		mvi 	a,1fh     	; 8085-RST5.5 disable
		mvi 	b,00h     	; NSC800-RSTC disable
intcte:	sim
		nop 	          	; fuer NSC800-JR-Befehl
		mov 	a,b
		out 	0bbh
		pop 	b
		ret
;
secwrx:	call	init
		push	h
		lxi 	h,intwr
		shld	intprg
		pop 	h
		lda 	trknr
		mov 	d,a
		mvi 	a,2ah
		cmp 	d         	; Track kleiner 42 ?
		mvi 	a,0a2h	; keine Preecomp
		jnc 	write1
		mov 	b,a
		lda 	selbyt
		ani 	01h
		mov 	a,b
		jnz 	write1
		ani 	0fdh
write1:	sta 	rwbef
		jmp 	start
;
;
;
secrdx:	call	init
		push	h
		lxi 	h,intrd
		shld	intprg
		pop 	h
		mvi 	a,84h		;rdsec;+30 ms
		sta 	rwbef
;
start:	mov	a,m
		rlc
		cnc	recal
		mov	a,m
		ani	7fh
		out	trkreg
		lda	trknr
		out	datreg
		mvi	b,14h		; Seek
		call	posit
		ani	19h		; Verify
		jnz	end0
		in 	trkreg
		mov	d,a
		lda	trknr
		cmp	d
		jnz	poserr
		xra	a
		cmp	b
		mov	a,c
		rz
		in 	trkreg	; FDC-Befehl wurde mit Fehler
		ori	80h		; abgeschlossen
		mov	m,a
		mvi 	b,4h
wdhg:		push	b
		push	h
		call	rdwr
		ani 	5ch
		pop 	h
		pop 	b
		jz  	ende
		dcr 	b
		jnz 	wdhg

end0:		mvi 	m,0h
ende:		in  	status
		ani 	7ch
end1:		push	psw
;
		call	intof
		pop 	psw
		ret
;
recal:	mvi 	b,04h		; Restore
		call	posit
		xra 	a
		cmp 	b
		jz  	recal1
		mvi 	m,80h
		in  	status
		ret
;
recal1:	mov	a,c
		ret
;
;
;
init:		di
		shld	datadr
		push	b
		mov 	a,d
		sta 	trknr
		mov 	a,e
		sta 	secnr


		lxi 	h,0024h
		mvi 	m,0e9h	; TRAP :   PCHL
		lxi 	h,0066h
		mvi 	m,0e9h

		lxi 	h,002ch
		mvi 	m,0f1h	; RST5.5:  POP  PSW
		inx 	h
		mvi 	m,0dbh	;          IN
		inx 	h
		mvi 	m,048h	;          48h
		inx 	h
		mvi 	m,0c9h	;          RET
		mov 	a,b
		sta 	selbyt
		rrc
		ani 	03h
		push	psw
		lxi 	h,laufw0
		mvi 	d,0h
		mov 	e,a
		dad 	d
		mov 	c,m
		lda 	selbyt
		ani 	21h
		ora 	c
		sta 	selout
		lxi 	h,track0
		pop 	psw
		mvi 	d,0h
		mov 	e,a
		dad 	d
		pop 	b
		ret
;
rdwr:		lhld	datadr
		push	h
		pop 	d
		lhld	intprg
		lda 	secnr
		out 	secreg
		call	inton

		lda 	rwbef
		out 	comreg
rdwr1:	ei
		hlt
		jmp 	rdwr1
;
;
;Zuordnung log. zu phy.Laufwerke
;
laufw0:	db   02h	; = A CF
laufw1:	db   04h    ; = B SS Moppel Format
laufw2:	db   08h    ; = C DS ITT3030 Format
laufw3:	db   08h    ; = D IDE
;
;
posit:	call	inton
;
		lda	selbyt
		rrc
		rrc
		rrc
		ani	03h
		ora	b
		out	comreg
posit1:	in	status
		rlc
		jnc	posit1
		lda	selout
		out	ctrl
		ei
		mvi	b,2h
posit2:	lxi	d,0ffffh
posit3:	dcx	d
		mov	a,d
		ora	e
		jnz	posit3
		dcr	b
		jnz	posit2
		mvi	a,0d0h
		out	comreg
		call	intof

		in  	status
		ani 	7ch
		ori 	80h
		mov 	c,a
		ret
;
poserr:	mvi	m,0h		;Posit.-Fehler
		mvi	a,10h
		ret
;
;
secrd:	lda	0066h
		sta	nscbuf
;
vsecrd:	shld	hlbuff
;
		lxi 	h,0h
		dad 	sp
		shld	spbuff
		lxi 	sp,stack2
		lhld	hlbuff

		call	secrd0
		shld	hlbuff
		lhld	spbuff
		sphl
		lhld	hlbuff
		push	psw
		lda 	nscbuf
		sta 	0066h
		pop 	psw
		ret
;
;
;
secrd0:	mvi	c,10h
secrdl:	push	h
		push	d
		push	b
		call	secrdx
		pop 	b
		pop 	d
		pop 	h
		ora 	a
		rz
		dcr 	c
		jnz 	secrdl
		inr 	a
		ret
;
secwr:	lda 	0066h
		sta 	nscbuf
		mvi 	c,10h
vsecwr:	nop
secwrl:	push	h
		push	d
		push	b
		call	secwrx
		pop 	b
		pop 	d
		pop 	h
		ora 	a
		jz  	verify
		dcr 	c
		jnz 	secwrl
		inr 	a
		push	psw
		lda 	nscbuf
		sta 	0066h
		pop 	psw
		ret
;
;
verify:	push	h
		shld	vbufhl
;
verxx:	push	d
		push	b
		lxi 	h,verbuf
		call	vsecrd
		lhld	vbufhl
		call	compar
		pop 	b
		pop 	d
		pop 	h
		jnz 	vsecwr
		lda 	nscbuf
		sta 	0066h
		xra 	a
		ret
;
compar:	mvi 	b,0h
		lxi 	d,verbuf
compl:	ldax	d
		cmp 	m
		rnz

		inx 	h
		inx 	d
		dcr 	b
		jnz 	compl
		ret
;
;

;####################################################################
;
; CF Interface
;
;
;--------------------------------------------------------------------
;
; Befehle fuer AVR
;
; bcf...	Befehl CF-Karte xy
;
;
bcfstr	equ	10h		; Spurnummer setzen
bcfssec	equ	20h   	; Sektornummer setzen
bcfrds	equ	30h		; Daten vom aktuellen Sektor lesen
bcfwrs	equ	40h		; Daten in den aktuellen Sektor schreiben
;
bcfres	equ	00h		; AVR-Reset
bcfst		equ	0f0h		; AVR-Status
bcfstart	equ	50h		; RTS loeschen
bcfstop	equ	51h		; RTS setzen
bcftx		equ	52h		; ein byte senden		ohne Buffe
bcfrx		equ	53h		; ein byte empfangen
bcfrxs	equ	55h		; Zeile empfangen bis CR (max 64Zeichen)
bcftxz	equ	56h		; ein Byte senden (mit Timeout 10s)
bcfrxz	equ	57h		; ein byte empfangen mit Timeout 10s
bcfvst 	equ	5Fh		; Status der V24 Schnittstelle
					; Bit 3 CTS Signal
					; Bit 4 RTS Signal
					; Bit 5 Senderegister leer
					; Bit 7 Empfangsregister voll
;
;--------------------------------------------------------------------
; CF-Karte nur noch V24
;
; PIO-A lesen (A)
; wartet bis Daten bereitstehen

piord:	in	cf_c		; Status ermitteln
		ani	00100000b	; IBF maskieren
		jz	piord		; warte auf Daten
		in	cf_a		; daten lesen
		ret			; (A) Daten
;
; (a) pio-a schreiben
;

piowr:	push	psw		;
pio_sts:	in	cf_c		; Status ermitteln
		ani	10000000b	; /OBF maskieren
		jz	pio_sts	; warten auf Quittung /ACK
		pop	psw
		out	cf_a		; Daten schreiben
		ret			;
;
;====================================================
;
; PIO Initialisieren mit Betriebsart
; Kanal A bidirektional
; Kanal C Handshake
; Kanal B Ausgabe
;
pioinit:	push	psw		; PIO Betriebsart einstellen
		mvi	a,cf_ba	; PIO_BA	.equ	0c1h
		out	cf_ctl	;
		pop	psw		;
		ret			;
;
;
;
;####################################################################
; Modul fuer IDE-Karte
;
; Entwickelt aus IDETOOL von Alfred Klichs 2022,;
; Paul's 8051 Code Library, Paul Stoffregen 2005
; https://www.pjrc.com/tech/8051/ide/
;
;
; Assembler 8085-Simulator-ID V4.93 Oshonsoft (Vladimir Soso)
; Editor 	Notepad++ TAB=6
;
;
; Stand:
;	24.01.2023	aus FDC-0123 vom 23.01.2023
;
;--------------------------------------------------------------------
;
; IO-Adressen fuer PIO 8255
;
PIO_IDE		EQU	058H		; PIO I/O Adresse
;
PIO_LSB		EQU	0 + PIO_IDE ; Port A
PIO_MSB		EQU	1 + PIO_IDE ; Port B
PIO_CTL		EQU	2 + PIO_IDE ; Port C
PIO_CFG		EQU	3 + PIO_IDE ; I/O modus
;
PIO_RDW		EQU	10010010B 	; Port A+B input, Port C output
PIO_WR		EQU	10000000B 	; Port A+B+C output
;
IDE_DRV		EQU	11100000B 	; select the device
			; x1x00000b LBA master
			; x1x10000b LBA slave
			; x0x00000b CHS master
			; x0x10000b CHS slave
;			; Kanal B als Ausgabe
;
;--------------------------------------------------------------------
;
; Variable fuer IDE Funktionen
;
IDE_A0		EQU	00000001B 	; A0 direct to IDE HDD
IDE_A1		EQU	00000010B 	; A1 dto.
IDE_A2		EQU	00000100B 	; A2 dto.
IDE_CS0		EQU	00001000B 	; CS0 to Inverter
IDE_CS1		EQU	00010000B 	; CS1 dto.
IDE_WR		EQU	00100000B 	; WR dto.
IDE_RD		EQU	01000000B 	; RD dto.
IDE_RST		EQU	10000000B 	; RST dto.
;
IDE_DATA		EQU	IDE_CS0
IDE_ERR		EQU	IDE_CS0 + IDE_A0
IDE_SEC_CNT		EQU	IDE_CS0 + IDE_A1
IDE_SECTOR		EQU	IDE_CS0 + IDE_A1 + IDE_A0
IDE_CYL_LSB		EQU	IDE_CS0 + IDE_A2
IDE_CYL_MSB		EQU	IDE_CS0 + IDE_A2 + IDE_A0
IDE_HEAD		EQU	IDE_CS0 + IDE_A2 + IDE_A1
IDE_COMMAND		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_STATUS		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_CONTROL		EQU	IDE_CS1 + IDE_A2 + IDE_A1
IDE_ASTATUS		EQU	IDE_CS1 + IDE_A2 + IDE_A1 + IDE_A0
;
IDE_CMD_RECAL	EQU	010H
IDE_CMD_READ	EQU	020H
IDE_CMD_WRITE	EQU	030H
IDE_CMD_INIT	EQU	091H
IDE_CMD_ID		EQU	0ECH
IDE_CMD_SPINDN	EQU	0E0H
IDE_CMD_SPINUP	EQU	0E1H
;
;
;===========================================================
;
; IDE/cf-karte lesen/schreiben
; laufwerk 2 = IDE/cf_karte
;
; parameteruebergabe:
;
; trknr		= spur	! Grenze 256Track
; secnr  		= sektor
; datadr 		= buffer zieladresse
; bufscr (C)	= 2 sektoren
;
;
; CHS_LBA uebertraegt die CHS zu LBA
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
;
;===========================================================
;
; IDE-karte lesen/schreiben
;  Laufwerk 0 = IDE-Karte
;
; Datenuebergabe:
;
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec 1.Sektor in Moppelformat
;			 wird im AVR-Teil auf 512B/Sektor umgerechnet
;
;	wrtflg wird auf 0 gesetzt Buffer ungueltig
;
; Parameteruebergabe:
;	datadr	= aktuelle Bufferadresse
;	trknr		= laufende Spurnummer
;	secnr		= laufende Sektornummer
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
; Rueckgabe:
;	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 fuer OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
;--------------------------------------------------------------------
;
; initrw =	Buffer, Spur, Sektorregister vorbereiten
;
; CHS_LBA uebertraegt die CHS zu LBA
;
; Parameteruebergabe:
;
;	datadr	= aktuelle Bufferadresse
;
;	LBA		= Sektor
;	LBA+1		= Track lsb
;	LBA+2		= Track msb
;	LBA+3		= Head

Iinitrw:	xra  	a		; clear wrtflg -"Buffer ungueltig"
		sta  	wrtflg	;
		di			; Interrupt sperren
		sta	lba+2
		sta	lba+3		; Head 0
		lxi	h,buffer	;
		shld	datadr	;
		lhld	akttrk	;
		shld	trknr		;
		shld	lba+1		; Track lsb/msb
		lda	aktsec	;
		sta	secnr		;
		sta	lba		; Sektor
		lxi	h,buffer	;
		shld	datadr	;
		ret			;
;
; get/put_cf =	Block von CF lesen/schreiben
;
get_ide:	pop	psw
		push	h
		push	d
		push	b
		call 	Iinitrw	;
		call 	READ_SECTOR	;	;
		ana	a		;
		jnz	Icf_err	; Fehler
		;
		call 	Iupdsek	; 2.Sektor laden
		call	READ_SECTOR	;
		ana	a		;
		jnz	Icf_err	; Fehler
		pop	b		; <== Rcksprung von Testroutine
		pop	d
		pop	h
		xra	a		;
		ret			; zurueck zum Disk-Manager
		;
		;
put_ide:	pop	psw
		push	h
		push	d
		push	b
		call 	Iinitrw	;
		call 	WRITE_SECTOR		;
		ana	a		;
		jnz	Icf_err	; Fehler
		;
		call 	Iupdsek	; 2.Sektor laden
		call	WRITE_SECTOR		;
		ana	a		;
		jnz	Icf_err	; Fehler
		pop	b
		pop	d
		pop	h
		xra	a		;
		ret			; zurueck zum Disk-Manager
;
;
; updsek/sp = Sektor und Spurnummern aktualisieren
;             und an CF-Karte senden
;
;	! maimal 32 Records pro Spur !
;
Iupdsek:	lxi	h,buffer+512
		shld	datadr
		lda	secnr		; Sektor holen
		cpi	16		; letzter Sektor spur
		jz	Iupdsp		;
		inr	a		; naechster Sektor gleiche Spur
		sta	secnr		;
		sta	lba
		ret

Iupdsp:	mvi	a,01h		; naechste Spur mit Sektor 1
		sta	secnr		;
		sta	lba
		lhld	trknr		;
		inx	h		;
		shld	trknr		;
		shld	lba+1		;
		ret
;
; Fehler (A)=1
;
Icf_err:	pop	b		;
		pop	d		;
		pop	h		;
		mvi	a,01		;
		stc			;
		ret			; Zurueck mit Fehler
;
;
;--------------------------------------------------------------------
;
ideinit:	call	IDE_HARD_RESET
		call	IDE_INIT
		jnz	nohd			; IDE Busy
		ret			;

nohd:		lxi	h,nomsg		; keine HD
		call	msgout		;
		ret

;
IDE_HARD_RESET:
		MVI	A, PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		MVI	A, IDE_RST		; hard reset drive
		OUT	PIO_CTL
		PUSH	B
		LXI	B,3000		; ca. 10 ms delay
WLOOP:
		DCX	B
		MOV	A,B
		ORA	C
		JNZ	WLOOP
		POP	B
		XRA	A			; no ide control lines asserted
		OUT	PIO_CTL
		RET
;
; HD Init
;
IDE_INIT:
		call	IDE_GET_ASTATUS	; Test ob Karte
		ana	a			; gesteckt ist
		jz	IDE_NOT_RDY		; nein --> Ende

		MVI	A,IDE_HEAD		;
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8		;
		LXI	D,0010h		; wait for ready
IDE_RDY_BSY:
		MVI	B,10h
INILP:					; delay for slow drives
		DCR	B
		JNZ	INILP			; djnz $ Z80
		DCX	D
		MOV	A,D
		ORA	E
		JZ	IDE_NOT_RDY		; timeout
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C

		ANI	01000000B		; bit 6,a
						; Z80, if on then ready
		JZ	 IDE_RDY_BSY	; wait for READY bit to be set
		MOV	A,C

		ANI	10000000B		; bit 7,a
						; Z80, if on then busy
		JNZ	 IDE_RDY_BSY	; wait for BUSY bit to be clear
		RET

IDE_NOT_RDY:
		MVI	a,0FFh
		ORA	A
		STC				; timeout
		RET

;
;
; Wait for DRQ
;
IDE_WAIT_DRQ:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_DRQ
MORE_DRQ:

		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	10001000B
		CPI	00001000B
		JZ	DONE_DRQ
		DCR	B
		JNZ	MORE_DRQ
		LDA	DELAY_DRQ
		DCR	A
		JNZ	MORE_DRQ
		STA	DELAY_DRQ
		STC
		RET
DONE_DRQ:
		XRA	A
		RET

DELAY_DRQ:
		DB	0

;
; Wait for BUSY to be clear
;
IDE_WAIT_NOT_BUSY:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_BUSY
MORE_BUSY:
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	11000000B
		XRI	01000000B
		JZ	NOT_BUSY
		DCR	B
		JNZ	MORE_BUSY
		LDA	DELAY_BUSY
		DCR	A
		STA	DELAY_BUSY
		JNZ	MORE_BUSY
		STC
		RET
NOT_BUSY:
		XRA	A
		RET

DELAY_BUSY:
		DB	0

;
; Get Active Status of the IDE Drive
;
IDE_GET_ASTATUS:
		MVI	A,IDE_ASTATUS
		CALL	IDE_RD_8
		MOV	A,C
		RET

;
; 8 bit read
;
IDE_RD_8:
		MOV	D,A			; save adress to D
		OUT	PIO_CTL		; drive address onto control lines
		ORI	IDE_RD		; set read pin on
		OUT	PIO_CTL
;		nop				; delay 2x nop
;		nop
		IN	PIO_LSB		; read lower byte
		MOV	C,A			; to C
		MOV	A,D			; reset read pin first
		OUT	PIO_CTL
		XRA	A			; reset all control pins
		OUT	PIO_CTL
		RET

;
; 8 bit write
;
IDE_WR_8:
		MOV	D,A			; save adress to D
						; Command, head, sector, cylinder etc. in C
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
		MOV	A,C
		OUT	PIO_LSB		; Drive lower lines
		MOV	A,D			; Drive address onto control lines
		OUT	PIO_CTL
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
;		nop				; delay	2x nop
;		nop
		MOV	A,D
		OUT	PIO_CTL		; reset write pin first
		XRA	A
		OUT	PIO_CTL		; reset all control pins
		MVI	A,PIO_RDW
		OUT	PIO_CFG		; set PIO read/write mode
		RET

;
; Do the identify command and fill
; the buffer with the drive infos
;
DRIVE_ID:
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_HEAD
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_COMMAND
		MVI	C,IDE_CMD_ID	; 0ech
		CALL	IDE_WR_8		; issue the command
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		RET

;
; 16 bit read
;
READ_SECTOR:
						; hdread:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_READ
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
		JMP	GET_ERR

;
; 16 bit write
;
WRITE_SECTOR:
						; hdwrite:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_WRITE
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	WRITE_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
						;
						; Return with errors if occurs
GET_ERR:
		MVI	A,IDE_ERR
		CALL	IDE_RD_8
		MOV	A,C
		ORA	A			; set Zero Flag
		RET

;
;
; Read one sector of 512 bytes
;




READ_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	datadr		; target
READ_LP:
		MVI	A,IDE_DATA		; drive address onto control lines
		OUT	PIO_CTL		; and set read pin off (loop)
		ORI	IDE_RD
		OUT	PIO_CTL		; set read pin on
		IN	PIO_LSB		; read the lower byte
		MOV	M,A
		INX	H
		IN	PIO_MSB		; read the upper byte
		MOV	M,A
		INX	H
		DCR	B			; Z80 djnz read_lp
		JNZ	READ_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL		;
		shld	datadr		; Zeiger sichern
		ret
;
; Write one sector of 512 bytes
;
WRITE_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	datadr		; target			; source
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
WRITE_LP:
		MOV	A,M
		OUT	PIO_LSB		; Drive lower lines with lsb
		INX	H
		MOV	A,M
		OUT	PIO_MSB		; Drive upper lines with msb
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; Drive address onto control lines
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; reset write pin
		INX	H
		DCR	B			; Z80 djnz write_lp
		JNZ	WRITE_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL
		MVI	A,PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		shld	datadr		; Zeiger sichern
		RET
;
;
; LBA = 28bit 4+3x8
; If CHS mode head 8-bit (0-254), sektor 6-bit (1-63)
; cylinder/track 10-bit (0-1023) = 24-bit / 8,38GB max.
;
;
WR_LBA:	LDA	LBA+3			; 4/8-bit, CHS head 0-254
		ANI	0FH			; LBA 0-15
		ORI	IDE_DRV		; mode master/slave CHS/LBA
		MOV	C,A
		MVI	A,IDE_HEAD
		CALL	IDE_WR_8

		LDA	LBA+2			; 8-bit, CHS cyl msb
		MOV	C,A
		MVI	A,IDE_CYL_MSB
		CALL	IDE_WR_8

		LDA	LBA+1			; 8-bit, CHS cyl lsb
		MOV	C,A
		MVI	A,IDE_CYL_LSB
		CALL	IDE_WR_8

		LDA	LBA			; 8-bit, CHS sector

;		and	3fh			; 1-63 if CFC/CHS
		MOV	C,A
		MVI	A,IDE_SECTOR
		CALL	IDE_WR_8

		MVI	C,1			; transfer one sector
		MVI	A,IDE_SEC_CNT
		CALL	IDE_WR_8
		XRA	A
		RET
;
;
;
;--------------------------------------------------------------------
;
;
NOMSG:	DB	"No HDD found!"
		db	0ah		;
		db	0dh		;
		db	00h		;


;
;####################################################################

;--------------------------------------------------------------------
;
; V24 Schnittstelle CF-Karte
;
; Byte ueber V24 Senden
; (C) an CF-Karte
cv24txb:	mvi  	a,bcftx	; Befehl
		call 	piowr		;
		mov  	a,c		; Zeichen
		call 	piowr		; ausgeben
		ret			;
;
; Byte von V24 holen
; (A) von CF-Karte
cv24rxb:	mvi  	a,bcfrx	; Befehl
		call 	piowr		;
		call 	piord		; Zeichen holen
		ret
;
;
; Byte ueber V24 Senden (mit Timeout)
; (C) an CF-Karte
cv24txbz:	mvi  	a,bcftxz	; Befehl
		call 	piowr		;
		mov  	a,c		; Zeichen
		call 	piowr		; ausgeben
		ret			;
;
; Byte von V24 holen mit Timeout nach 10s
; (A) von CF-Karte
cv24rxbz:	mvi  	a,bcfrxz	; Befehl
		call 	piowr		;
		call 	piord		; Zeichen holen
		ret
;
; V24 Statusabfrage 	ost= OutputStatus
;			ist= InputStatus
;
; (A)=00h Schnittstelle belegt
; (A)=FFh Schnittstelle frei
cv24ost:	mvi  	a,bcfvst	; Status fr Ausgabe
		call 	piowr		; abfragen
		call 	piord		;
		ani  	00100000b	; Bit 5 maskieren
		jz   	cv24oe	; kein Zeichen vorhanden
		mvi  	a,0ffh	; Zeichen im Eingangsbuffer
cv24oe:	ret
;
;
; (A)=00h kein Zeichen
; (A)=FFh Zeichen wartet
cv24ist:	mvi  	a,bcfvst	; Status fr Eingabe
		call 	piowr		; abfragen
		call 	piord		;
		ani	10000000b	; Bit 7 Maskieren
		jz	cv24ie	; kein zeichen
		mvi	a,0ffh	; Zeichen im Eingangsbuffer
cv24ie:  	ret			;

;
;####################################################################
;
; Multi-IO-Karte initialisieren
;
mioini:	mvi	a,bap1	; PIO-Betriebsarten einstellen
		out	pio1s		;
		mvi	a,bap2	;
		out	pio2s		;
		xra	a		;
		out	pio2b		;
		out	pio2c		;
		;
		mvi	a,03h		; Reset com1 1
		out	com1s		;
		mvi	a,bacom1	; com1-Betriebsart
		out	com1s		;
		mvi	a,55h		; RST On
		out	com1s		;
		;
		mvi	a,03h		; Reset com2 2
		out	com2s		;
		mvi	a,bacom2	; com2-Betriebsart
		out	com2s		;
		mvi	a,55h		; RST On
		out	com2s		;
		;
		mvi	a,bati0	; Timer-Betriebsarten
		out	tims		;
		mvi	a,bati1	;
		out	tims		;
		mvi	a,bati2	;
		out	tims		;
		;
		lxi	d,baud	; siehe oben
		mov	a,e		; in Timer 0 laden
		out	tim0		;
		mov	a,d		;
		out	tim0		;
		;
		mov	a,e		; in Timer 1 laden
		out	tim1		;
		mov	a,d		;
		out	tim1		;
		;
		mov	a,e		; in Timer 2 laden
		out	tim2		;
		mov	a,d		;
		out	tim2		;
		;
		call	irqi		; Interuppt-Service-Routine einstellen
		;
		ret
;
;--------------------------------------------------------------------
;
; Baudrate einstellen
; (DE)= Tabellenwert fuer 6Mhz
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	     01h highByte
;
com1bd:	mov	a,e		; in Timer 0 laden
		out	tim0		;
		mov	a,d		;
		out	tim0		;
		ret
		;
com2bd:	mov	a,e		; in Timer 1 laden
		out	tim1		;
		mov	a,d		;
		out	tim1		;
		ret
		;
;
; V24 Input-Status	(A)=00h kein Zeichen
;				(A)=FFh Zeichen vorhanden
;
com1ist:	mvi 	a,15h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS off
		in 	com1s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz 	com1i_e	; Zeichen vorhanden
		mvi 	a,55h		; RTS on
		out 	com1s		;
		xra 	a		;
		ret			;

com1i_e:	mvi 	a,55h		; RTS on
		out 	com1s		;
		mvi 	a,0ffh	;
		ret			;
;
com2ist:	mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
		in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz 	com2i_e	; Zeichen vorhanden
		mvi 	a,55h		; RTS on
		out 	com2s		;
		xra 	a		;
		ret			;

com2i_e:	mvi 	a,55h		; RTS on
		out 	com2s		;
		mvi 	a,0ffh	;
		ret			;
;
; Byte v24 senden (C)
; wartet bis Empfaenger bereit ist
;
;
com1t:	mvi 	a,55h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS on
com1t_l:	in 	com1s		; Warten bis Senderegister leer
		ani 	02h		;
		jz	com1t_l	;
		mov	a,c		;
		out 	com1d		; Daten senden
		ret			;
;
com2t:	mvi 	a,55h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS on
com2t_l: 	in 	com2s		; Warten bis Senderegister leer
		ani 	02h		;
		jz	com2t_l	;
		mov	a,c		;
		out 	com2d		; daten senden
		ret			;
;
;
;--------------------------------------------------------------------
;
; Byte von V24 empfangen (A)
; wartet bis Zeichen vorhanden ist
;
com1r:	mvi 	a,15h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com1r_l:	in 	com1s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jz  	com1r_l	; warten
		mvi 	a,55h		; RTS on
		out 	com1s		;
		in 	com1d		; Daten auslesen
		ret			;
;
com2r:	mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com2r_l:	in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jz  	com2r_l	; warten
		mvi 	a,55h		; RTS on
		out 	com2s		;
		in 	com2d		; Daten auslesen
		ret			;
;
; Byte von V24 empfangen
; Abbruch bei Timeout
;
timeout	equ	0800h		; ca. 2s
;
com1a:	push	b		;
		lxi	b,timeout	;
		mvi 	a,15h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com1a_l:	in 	com1s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz	com1a_e	;
		call	dely1		;
		dcr	c		; Timeout ?
		jnz	com1a_l	;
		dcr	b		;
		jnz  	com1a_l	; warten
		pop	b		;
		mvi	a,1ah		;
		ret			;
;
com1a_e:	mvi 	a,55h		; RTS on
		out 	com1s		;
		in 	com1d		; Daten auslesen
		pop	b		;
		ret			;
;
com2a:	push	b		;
		lxi	b,timeout	;
		mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com2a_l:	in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz	com2a_e	;
		call	dely1		;
		dcr	c		; Timeout ?
		jnz	com2a_l	;
		dcr	b		;
		jnz  	com2a_l	; warten
		pop	b		;
		mvi	a,1ah		;
		ret			;
;
com2a_e:	mvi 	a,55h		; RTS on
		out 	com2s		;
		in 	com2d		; Daten auslesen
		pop	b		;
		ret			;
;
;--------------------------------------------------------------------
;
; Com1rx 	Daten aus FIFO lesen
;
; (A)=Zeichen
; (A)=0 und Z=1 Timeout
;
maxbuf	equ 3fh		; Bufferlaenge
rtsbuf	equ 2ah		; Bufferposition fuer RTS
;
; buffz	dw	0000h	; Schreibzeiger fuer Zielbuffer
; wrz		dw	0000h	; Schreibzeiger fuer IRQ
; rdz		dw	0000h	; Lesezeiger
; v24flag	db	00h	; 1001  0001
;				; :  :     :
;				; :  :     +-- RTS punkt erreicht
;				; :  +-------- INIT Flag 1=IRQ,etc. OK
;				; :                      0=Init IRQ,etc.
;				; :                      durchfuehren
;				; +---------- Buffer voll
;
com1rx:	push	b		;
		push	d		;
		push	h		;
		lda	v24flag	;
		ani	10h		; Init-Flag ?
		cz	setirq	; nein -> Zeiger auf Anfang
rxz_l1:	lxi	b,timeout	; Abbruchbedingung laden
rxz_l2:	di			;
		lhld	wrz		; daten in FIFO ?
		xchg			;
		lhld	rdz		;
		mov	a,l		;
		cmp	e		;
		jc	rxzrb		; ja --> Daten lesen
		lda	v24flag	;
		ani	01h		; RTS gesetzt ?
		jnz	rtscl		; RTS und IRQ freigeben
		ei			;
		lda	v24flag	;
		ani	080h		; uffer Ende ?
		jnz	rxz_e		; ja --> Ende mit Bufferueberlauf
		call	dely1		;
		dcr	c		; Timeout ?
		jnz	rxz_l2	;
		dcr	b		;
		jnz  	rxz_l2	; warten
		xra	a		; keine Daten im FIFO (A)=0 Z=1

rxz_e:	di			; IRQ sperren
		pop	h		;
		pop	d		;
		pop	b		;
		ret			; Ende RX

;
;
; setirq	IRQ, UFFER, Flags setzen
;
setirq:	lxi	h,zbuff	;
		shld	wrz		;
		shld	rdz		; auf Anfang
		mvi	a,10h		; INIT-Flag setzen
		sta	v24flag	; Init-Flag setzen
		ret			;
;
; Buffer auslesen
;
rxzrb:	di
		mov	a,m		; Byte holen
		inx	h		;
		shld	rdz		;
		jmp	rxz_e		; warten auf naechstes Byte

;
; RTS und IRQ freigeben
;
rtscl:	di			;
		lxi	h,zbuff	; Buffer
		shld	wrz		;
		shld	rdz		; auf Anfang
		lda	v24flag	;
		ani	0feh		; 1111 1110b
		sta	v24flag	; RTS-Flag loeschen
		mvi	a,95h		; 1001 0101 = RX IRQ und
		out	com1s		; RTS freigeben
		ei
		jmp	rxz_l1	;
;
;
;--------------------------------------------------------------------
;
; IRQ vorbereiten
;
irqi:		lxi	h,int75	; Spungtabelle
		mvi	m,0c3h	; = JMP
		inx	h		; IRQ Seviceroutine
		lxi	d,irqs	; In Sprungtabelle
		mov	m,e		;
		inx	h		;
		mov	m,d		; eintragen
		mvi	a,1bh		; 0001 1011 irq7.5 freigeben
		sim			;
		ret			;
;
; IRQ Serviceroutine
; Ein Byte uber V24 empfangen und im Empfangsbuffer ablegen
;
irqs: 	di			; IRQ sperren
		push	psw		; alles sichern
		push	h		;
		push	d		;
		push	b		;
		in	com1s		;
		ani	01h		; Senderegister voll
		jz	irqe		; nein
		lhld	wrz		; Zeiger auf Zeilenbuffer
		in	com1d		;
		mov	m,a		; daten ablegen
		inx	h		;
		shld	wrz		; Zeiger sichern
		mvi	a,rtsbuf	; Zeiger an RTS Grenze
		cmp	l		;
		jz	setrts	; RTS Setzen
		mvi	a,maxbuf	; Test auf Ende Zeilenbuffer
		cmp	l		;
		jz	irqbs		; Zeilenbuffer Ende erreicht

irqe:		pop	b		; alles wieder zurueck
		pop	d		;
		pop	h		;
		pop	psw		;
		ei			; IRQ freigeben
		ret			;

setrts:	mvi	a,0d5h	; RTS gesetzt
		out	com1s		; IRQ Freigabe
		lda	v24flag	;
		ori	01h		;
		sta	v24flag	; RTS-Flag setzen
		jmp	irqe		;

irqbs:	lxi	h,zbuff	; Buffer wieder auf Anfang
		shld	wrz		;
		mvi	a,55h		; RTS gesetzt
		out	com1s		; IRQ gesperrt
		lda	v24flag	;
		ori	81h		; 1000 0001 Error u. V24 Flag setzen
		sta	v24flag	;
		jmp	irqe		;
;
;####################################################################
;
; V24 Moppel 87er Hardware
;
; Baudratentabelle fuer
; CPU-Frequenz 6,144MHz
;
; Teilfaktor fuer Timer = CPU-Frequenz /2 = Systemtakt / 4800bd / 16 = 40
;                                                                  I
;                                                                  UART Teiler
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;               sonst Umrechnung nach Binaer notwendig
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	     01h highByte
;
; (DE) Baudrate laden
;
mv24ini:	lxi	d,0020h	; Startwert 9600Bd
;
; Baudrate laden (DE)
;
mv24bd:	mvi	a,37h		; Ctl Zaehler als BCD-Zaehler
		out	timer_0s	;
		mov	a,e		;
		out 	timer_00	; Zaehler untere haelfte
		mov	a,d		;
		out 	timer_00	; Zaehler obere haelfte
		mvi 	a,48h		; v24 modus
		out 	cs_mode	;
		mvi 	a,03		; Reset UART
		out 	com0s		;
		ret
;
; V24 Input-Status	(A)=00h kein Zeichen
;				(A)=FFh Zeichen vorhanden
;
mv24ist:	mvi 	a,15h		;
		out 	com0s		; clk/16,8bit,1stop,ohne paritaet , RTS off
mvist_l:	in 	com0s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz 	mvist_e	; Zeichen vorhanden
		mvi 	a,55h		; RTS on
		out 	com0s	;
		xra 	a		;
		ret			;

mvist_e:	mvi 	a,55h		; RTS on
		out 	com0s	;
		mvi 	a,0ffh	;
		ret			;
;
; Byte von V24 empfangen (A)
; wartet bis Zeichen vorhanden ist
;
mv24rx:	mvi 	a,15h		;
		out 	com0s		; clk/16,8bit,1stop,ohne paritaet , RTS off
mvrx_l:	in 	com0s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jz  	mvrx_l	; warten
		mvi 	a,55h		; RTS on
		out 	com0s		;
		in 	com0d		; Daten auslesen
		ret			;
;
; Byte v24 senden (C)
; wartet bis Empfaenger bereit ist
;
mv24tx:	mvi 	a,55h		;
		out 	com0s		; clk/16,8bit,1stop,ohne paritaet , RTS on
mvtx_l:	in 	com0s		; Warten bis Senderegister leer
		ani 	02h		;
		jz	mvtx_l	;
		mov	a,c		;
		out 	com0d		;
		ret			;
;
; Byte V24 empfangen (A)
; mit Zeitueberwachung 10s (A)=1ah Control Z
;
mv24rxz:	push 	b		; ein Byte ueber V24 empfangen (A)
		lxi 	b,10000	; max Wartezeit 10s
		mvi 	a,15h		;
		out 	com0s		; clk/16,8bit,1stop,ohne paritaet , RTS off
mvrxz_l:	in 	com0s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz  	mvrxz_e	;
		call 	dely1		; 1ms warten
		dcr	c		;
		jnz 	mvrxz_l	;
		dcr	b		;
		jnz	mvrxz_l	;
		pop 	b		;
		mvi	a,55h		; RTS on
		out	com0s	;
		mvi 	a,1ah		; SUB (Control-Z)
		ret

mvrxz_e:	mvi	a,55h		; RTS on
		out	com0s		;
		in	com0d		; Daten auslesen
		pop	b		;
		ret			;
;
; Byte V24 senden (C)
; mit Zeitueberwachung
;
mv24txz:	mvi 	a,55h		; ein Byte ueber V24 senden (C)
		out 	com0s		; clk/16,8bit,2stop,ohne paritaet RTS on
		mvi 	b,10		; max Wartezeit 10ms
mvtxz_l:	in 	com0s		; Warten bis Senderegister leer
		ani 	02h		;
		jnz  	mvtxz_o	;
		call 	dely1		;
		dcr 	b		;
		jnz 	mvtxz_l	;
		mvi 	a,04h		; Zeituebschreitung (End of Transmission)
		ret			;
	;
mvtxz_o:	mov 	a,c		; Daten ausgeben
		out 	com0d		;
		ret			;
;
;
;####################################################################
;
; Uhrzeit aus Monitor-Rot holen
; Rueckgabe:
; 	(HL) = Zeiger ASCII-Buffer
;
rdtime:	lxi	h,0h
		dad	sp        	; Bank 1 Stack in H&L
		lxi	d,stack0
		xchg
		sphl           	; Bank 0 Stack
		mvi	a,81h		;
		out	24h		; Bank0 Monitor zuschalten
		push	d         	; Bank 1 Stack retten
		;
		call	time		; Uhrzeit lesen (Monitorprogramm)
		mvi	c,15h		; Byte-Zaehler
		lxi	h,timbuf	; Lesezeiger
		lxi	d,tibuf	; Schreibzeiger
rdtim_l:	mov	a,m		; Daten von Monitor
		stax	d		; nach BIOS-Buffer kopieren
		inx	h		;
		inx	d		; naechste Adresse
		dcr	c		;
		jnz	rdtim_l	; weiter
		mvi	a,24h		; "$"
		stax	d		; Stringabschluss fuer CPM
		;
		pop	h		;Bank 1 Stack zurueck
		mvi	a,1h
		out	24h		;Umschalten auf Bank 1
		sphl			;alter Bank 1 Stack
		;
		lxi	h,tibuf	; Zeiger auf Bufferanfang stellen
		;
		ret
;
; Uhrzeit stellen (HL)Std,min, (BC)Wochentag, Kalendertag (DE)Monat,Jahr
;
wrtime:	ret			; in Arbeit
;
;
;-----------------------------------------------------------------------------
;
; Zeitverzoegerung 1ms, 10ms, 100ms 1sec
;
onesec:	push 	psw		; 1s warten
		mvi 	a,10
one_l:	call 	dely100
		dcr 	a
		jnz 	one_l
		pop 	psw
		ret
;
dely100:	push 	psw		; 100ms warten
		mvi 	a,100
d100_l:	call 	dely1
		dcr 	a
		jnz 	d100_l
		pop 	psw
		ret
;
dely10:	push 	psw		; 10ms warten
		mvi 	a,10
d10_l:	call 	dely1
		dcr 	a
		jnz 	d10_l
		pop 	psw
		ret
;
dely1:	push 	psw		; 1ms warten
		mvi 	a,06h
		add 	a
		add 	a
		inr 	a
		add 	a
		add 	a
		add 	a
d1_l: 	dcr 	a
		jnz 	d1_l
		pop 	psw
		ret
;
;-----------------------------------------------------------------------------
;
; BIOS-Meldetext
;
msg1:  	db	0ah
		db	0dh
		db	"This is 60 K"
		db	" CP/M Rel.2.2"
		db	0ah
		db	0dh
		db	"BIOS CMOP-244-V1 "
		db	"25.04.2024"
		db	0ah
		db	0dh
		db	00h
;
; Parameterbereich Multi-IO
;
wrz:		dw	0000h		; Schreibzeiger fuer IRQ
rdz:		dw	0000h		; Lesezeiger
v24flag:	db	00h		; 1001 0001
zbuff:	ds	30h		; FIFO
;
tibuf:	ds   10h		; Buffer fuer Uhrzeit (ASCII)
;
;
;=============================================================================
;
; Soft-System
;
		org	0F800h	;
;
;
trknr: 	dw   0h		; fr Disk-Routinen
secnr: 	db   0h		;     	-- " --
selbyt:	db   0h		;	-- " --
selout:	db   0h		;	-- " --
;
track0:	db   80h       	; trk-reg undefiniert
track1:	db   80h       	;     --  "  --
track2:	db   80h       	;     --  "  --
track3:	db   80h       	;     --  "  --
;
rwbef: 	db   0h		; fr Disk-Routinen
datadr:	dw   0h		; 	-- " --		Bufferadresse
intprg:	dw   0h		; 	-- " --
;
iodrv: 	db   0h		; Laufwerksnummer aus seldsk
iotrk: 	dw   offset		; <==    BDOS
iosec: 	db   1h		;	-- " --
iodma: 	dw   buff		;	-- " --
;
wrtflg:	db   0h		; 0= Buffer ungltig, Daten nachladen
aktdrv:	db   0h		; Laufwerk
akttrk:	dw   0h		; Track nach Blocking/Deblocking
aktsec:	db   0h		; Sektor -- " --
;
nscbuf:	db   0h		; Zwischenspeicher fr RAM 0066h
;			  		  wg. Floppy-Routine TRAP
hlbuff:	dw   0h		; Zwischenspeicher fr STACK-Umschaltung
spbuff:	dw   0h		; Zwischenspeicher fr BDOS-STACK
vbufhl:	ds   2h		;
;
dirbuf:	ds   128  		; Directory Access Buffer
alv0:  	ds   40
csv0:  	ds   40
alv1:  	ds   40
csv1:  	ds   40
alv2:  	ds   50
csv2:  	ds   50
alv3:  	ds   256		; fr 2048 Blcke a/4096 Byte
csv3:  	ds   0
;
LBA:		ds	4		; fuer IDE-Karte
;
;=================================================
;	RAM frei bis
;
;	stack2  	0FAFFh  ; bei 60 K
;
;	verbuf  	0FB00h  ; bei 60 K
;
;	buffer  	0FC00h  ; bei 60 K
;
;
end
;
